c
c
      program Diffusion1D
c
c
      PARAMETER (mnodex=1000)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
	character*60 fname
	character*7 flag, flagdummy
	character*30 flagvel1, flagvel2
	character*30 flagfld1, flagfld2
	character*30 flagfrm1, flagvelb1
c
c
	common /divtr/ xl, ctd, tinc, deltx
	common /divti/ ndivx, nnodex, nstep, nthd
	common /outpt/ iprint, ipnode
	common /filen/ nfidt, nfodt, nfthd
c
      dimension u0(mnodex)
c
c
c     xl       : length of rod
c     ctd      : coefficient for thermal diffusivity
c     ndivx    : number of division in x-axial
c     tinc     : time increment for calculation
c     nstep    : number of calculation steps
c     iprint   : skip number of output
c                results will be output at step = (j-1)*iprint   j=1, nstep
c     ipnode   : node number at which time history is output in 'nfodt'
c     nnodex   : number of node in x-axis; ndivx+1
c     mnodex   : maximum number of node in x-axis
c     deltx    : interval distance in x-axis; xl/(ndivx)
c     nfidt    : file number of input data
c     nfodt    : file number of output time-history data of a point 'ipnode' which is selected by  
c     nfthd    : file number of thermal distribution at iprint      
c     u0()     : array for tempurature at each node
c
c
      pi = 4.d0 * datan(1.d0)
c
      nnodex=mnodex
c
c 
c ---------------------------
c
c
c     / FILE NUMBER /
c
      nfidt=10      ! (nfidt, file='Diffusion1D.idt')
      nfodt=20      ! (nfodt, file='Diffusion1D.odt')
	nfthd=22      ! (nfidt, file='Diffsion1D_numebr.thd')
c
c     / OPEN FILES /
c
      call inpdat(u0)
c
c     / SOLVE /
c
      call solve_main(u0)
c
c
      stop 777
c
c
	end

c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   I N P D A T             ********
c*                                                                   *
c*********************************************************************
c
c
	subroutine inpdat(u0)
c
c     input parameters and initial conditions from (nfidt, file='Diffusion1D.idt')
c
      implicit double precision (a-h, o-z)
      implicit integer (i-n)
	character*5 flag
c
	common /divtr/ xl, ctd, tinc, deltx
	common /divti/ ndivx, nnodex, nstep, nthd
	common /outpt/ iprint, ipnode
	common /filen/ nfidt, nfodt, nfthd
c
      dimension u0(nnodex)
c
      pi = 4.d0 * datan(1.d0)
c
c
c     
c----------------------------------------------------------------------
c
c
      open (nfidt, file='Diffusion1D.idt')      !input data   ##
c
c
c     / INITIALIZATION OF ARRAY /
c
      do i=1, nnodex
          u0(i)=0.d0
      enddo
c
c     / PARAMETRS /
c
c
      rewind (nfidt)
  100 read (nfidt, 1000, end=555, err=999) flag
 1000 format(a5)
      if (flag .ne. '/para') goto 100
      read (nfidt, *) xl
      read (nfidt,*) ctd
      read (nfidt,*) ndivx
      read (nfidt,*) tinc
      read (nfidt,*) nstep
      read (nfidt,*) iprint
      read (nfidt,*) ipnode
      nnodex=ndivx+1
      deltx=xl/dble(ndivx)
	nthd=int(nstep/iprint)+1
c
      r=ctd*tinc/(deltx*deltx)
      if (r .ge. 0.5d0) then
	    write(*, 6000) r
 6000     format('r=', e15.7,
     *           ' / causion: tinc should be set to be smaller value')
      end if
c
c     / INITIAL VALUE OF u: u0 /
c
      rewind (nfidt)
  200 read (nfidt, 1000, end=555, err=999) flag
      if (flag .ne. '/data') goto 200
      do i=1, nnodex     
          read (nfidt, *) u0(i)
      enddo
	goto 777
c
c
  555 write(*, 5000)
 5000 format(' / flag required can not be found until end meet /')
      close (nfidt)
	stop 555
  999 write(*,9000)
 9000 format(' / Errors occur in reading / ')
      close (nfidt)
	stop 999
c
c
  777 close (nfidt)
      return
	end
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   S O L V E _ M A I N     ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine solve_main(u0)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
c
c
	common /divtr/ xl, ctd, tinc, deltx
	common /divti/ ndivx, nnodex, nstep, nthd
	common /outpt/ iprint, ipnode
	common /filen/ nfidt, nfodt, nfthd
c
c
      dimension u0(nnodex), u(nnodex), du(nnodex), ddu(nnodex)
      dimension time_ipnode(nthd)
	dimension u_ipnode(nthd), du_ipnode(nthd), ddu_ipnode(nthd)
c
c
      pi = 4.d0 * datan(1.d0)
c
c
c      u() : array of temprature
c     du() : array of change rate of temprature
c     ddu(): array of change acc. of temprature
c         r: coefficient of coductivity
c
c     -------------------------------------------
c
c
      r=ctd*tinc/(deltx*deltx)
c
c     / INITIALIZATION OF ARRAY /
c
      do i=1, nnodex
          u(i)=u0(i); du(i)=0.d0; ddu(i)=0.d0
	enddo
      do i=1, nthd
          time_ipnode(i)=0.d0; u_ipnode(i)=0.d0
		du_ipnode(i)=0.d0  ; ddu_ipnode(i)=0.d0
	enddo
c
c     / INITIAL CONDITION /
c
      istep=0; iopt=0
      tduration=0.d0
c
      call calc_u(r, u, du, ddu)
      do i=1, nnodex
          u(i)=u0(i)
	enddo
      call opt_thd(istep, tduration, u, du, ddu)
	iopt=iopt+1
      time_ipnode(iopt)=tduration
	u_ipnode(iopt)   =u(ipnode)
	du_ipnode(iopt)  =du(ipnode)
	ddu_ipnode(iopt) =ddu(ipnode)
c
c     / CALC. DIFFUSION /
c
      do i=1, nstep
	    istep=istep+1
          tduration=tduration+tinc
          call calc_u(r, u, du, ddu)
          call opt_thd(istep, tduration, u, du, ddu)
          if ( mod(istep, iprint) .eq. 0) then
c             write(*,6000) istep, nstep
c6000         format(' istep= ', i10, ' / ', ' nstep= ', i10)
	        iopt=iopt+1
              time_ipnode(iopt)=tduration
	        u_ipnode(iopt)   =u(ipnode)
	        du_ipnode(iopt)  =du(ipnode)
	        ddu_ipnode(iopt) =ddu(ipnode)
          endif
	enddo
c
c     / OUTPUT /
c
      call opt_odt(r, tduration,
     *             time_ipnode, u_ipnode, du_ipnode, ddu_ipnode)
c
c
c
      return
c
c
	end
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   C A L C _ U             ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine calc_u(r, u, du, ddu)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
c
c
	common /divtr/ xl, ctd, tinc, deltx
	common /divti/ ndivx, nnodex, nstep, nthd
	common /outpt/ iprint, ipnode
	common /filen/ nfidt, nfodt, nfthd
c
c
      dimension u(nnodex), du(nnodex), ddu(nnodex)
	dimension uu(nnodex), dummy(nnodex)
c
c
      pi = 4.d0 * datan(1.d0)
c
c     -------------------------------------------
c
c
c     / INITIALIZATION OF ARRAY /
c
      do i=1, nnodex
          uu(i)=0.d0
		dummy(i)=du(i)
	enddo
c
c     / CALC. DIFFUSION /
c
      do j=2, nnodex-1
          uu(j)=r*u(j-1)+(1.d0-2.d0*r)*u(j)+r*u(j+1)
      enddo
      do j=1, nnodex       ! update
          u(j)=uu(j)
      enddo
      do j=2, nnodex-1     ! update
	    du(j)=ctd*2.d0/(deltx*deltx)*( (u(j-1)+u(j+1))/2.d0-u(j) )
	enddo
	do j=1, nnodex       ! update
	    ddu(j)=( du(j)-dummy(j) )/tinc
	enddo
c
c
      return
c
c
	end
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   O P T _ T H D           ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine opt_thd(istep, tduration, u, du, ddu)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
	character*30 flagopt
      character*4 flagknd
	character*70 flag1, flag2
	character*9  flag3
	character*1  flag4
c
c
	common /divtr/ xl, ctd, tinc, deltx
	common /divti/ ndivx, nnodex, nstep, nthd
	common /outpt/ iprint, ipnode
	common /filen/ nfidt, nfodt, nfthd
c
c
      dimension u(nnodex)
      dimension du(nnodex), ddu(nnodex)
	dimension flag4(10)
c
c
      pi = 4.d0 * datan(1.d0)
c
c     -------------------------------------------
c
      flag4(1)='0';flag4(2)='1';flag4(3)='2';flag4(4)='3';flag4( 5)='4'
      flag4(6)='5';flag4(7)='6';flag4(8)='7';flag4(9)='8';flag4(10)='9'
c
c
      if (mod(istep, iprint) .ne. 0) goto 777
c
c
      flagopt='Diffsion1D_'       ! ##
      flagknd='.thd'
      in1=int(istep/100000000); mn1=mod(istep, 100000000)
      in2=int(mn1  /10000000 ); mn2=mod(mn1,   10000000 )
      in3=int(mn2  /1000000  ); mn3=mod(mn2,   1000000  )
      in4=int(mn3  /100000   ); mn4=mod(mn3,   100000   )
      in5=int(mn4  /10000    ); mn5=mod(mn4,   10000    )
      in6=int(mn5  /1000     ); mn6=mod(mn5,   1000     )
      in7=int(mn6  /100      ); mn7=mod(mn6,   100      )
      in8=int(mn7  /10       ); mn8=mod(mn7,   10       )
      flag3=trim(
     1  flag4(in1+1)//flag4(in2+1)//flag4(in3+1)//flag4(in4+1)//
     2  flag4(in5+1)//flag4(in6+1)//flag4(in7+1)//flag4(in8+1)//
     3  flag4(mn8+1)
     4           )
      flag2=trim(adjustl(flagopt))//trim(flag3)//trim(adjustl(flagknd))
      flag1=trim(adjustl(flag2))
c
	open (nfthd, file=flag1) 
c
c
      write (nfthd, 1000) istep, tduration
 1000 format(' / istep=', i5, ' / time=', e15.7)
      write (nfthd, 1020)
 1020 format('Node, x, u, du, ddu')
c
	deltx=xl/dble(ndivx)
      do i=1, nnodex
	    x=dble(i-1)*deltx
          write (nfthd, 2000) i, x, u(i), du(i), ddu(i)
 2000     format(i5, 3(e15.7, ','), e15.7)
      enddo
c
c
      close (nfthd)
c
c
  777 return
c
c
	end
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   O P T _ O D T           ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine opt_odt(r, tduration,
     *                   time_ipnode, u_ipnode, du_ipnode, ddu_ipnode)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
	character*30 flagopt
      character*4 flagknd
	character*70 flag1, flag2
	character*9  flag3
	character*1  flag4
c
c
	common /divtr/ xl, ctd, tinc, deltx
	common /divti/ ndivx, nnodex, nstep, nthd
	common /outpt/ iprint, ipnode
	common /filen/ nfidt, nfodt, nfthd
c
c
      dimension time_ipnode(nthd)
	dimension u_ipnode(nthd), du_ipnode(nthd), ddu_ipnode(nthd)
c
c
      pi = 4.d0 * datan(1.d0)
c
c     -------------------------------------------
c
c
	open (nfodt, file='Diffusion1D.odt') 
c
c
      write (nfodt, 1000) nstep, tduration, r
 1000 format(' / nstep=', i10, ' / duration time=', e15.7,
     *       ' / r= ', e15.7)
      write (nfodt, 1010) ipnode, dble(ipnode-1)*deltx, nthd
 1010 format(' / ipnode=', i10,  '/ xpnode=', e15.7, ' / nthd=', i10)
      write (nfodt, 1020)
 1020 format('time, u, du, ddu')
c
      do i=1, nthd
          write (nfodt, 2000) time_ipnode(i), 
     *           u_ipnode(i), du_ipnode(i), ddu_ipnode(i)
 2000     format(3(e15.7, ','), e15.7)
      enddo
c
c
      close (nfodt)
c
c
  777 return
c
c
	end
c
c
